'' =================================================================================================
''
''   File....... led_ctrl.spin
''   Purpose.... template file for PASM subroutines
''   Author..... Jon "JonnyMac" McPhalen (aka Jon Williams)
''               Copyright (c) 2010 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 
''   Updated.... 22 NOV 2010
''
'' =================================================================================================


var

  long  cog

  long  mstix
  long  cmd
  long  pin
  long  pulses
  long  timing
  
 
pub init | ok

  finalize

  ' set cog parameters

  mstix := clkfreq / 1_000                                              ' set ticks per millsecond
  cmd   := 0                                                            ' clear command
    
  ok := cog := cognew(@entry, @mstix) + 1                               ' start the cog

  return ok


pub finalize

'' Stops cog (if running)

  if cog
    cogstop(cog~ - 1)


pub set(p, state)

' sets pin p to state.0

  repeat while (cmd <> 0)                                       ' wait if presently busy
  pulses := state                                               ' set state
  pin := p                                                      ' set pin #
  cmd := 1                                                      ' set command


pub blink(p, blcount, ms)

' blinks pin p 

  repeat while (cmd <> 0)
  timing := ms                                                  ' set pulse timing
  pulses := blcount                                             ' set # of blinks
  pin := p                                                      ' set pin #
  cmd := 2                                                      ' set command


pub busy

' returns state of LED processor

  return (cmd <> 0)
  

dat

                        org     0

entry                   mov     tmp1, par                       ' start of structure
                        rdlong  ms001, tmp1                     ' read ticks per ms
                        add     tmp1, #4
                        mov     cmdpntr, tmp1                   ' save address of cmd
                        add     tmp1, #4
                        mov     pinpntr, tmp1                   ' save address of pin
                        add     tmp1, #4
                        mov     pulsepntr, tmp1                 ' save address of pulses
                        add     tmp1, #4
                        mov     timepntr, tmp1                  ' save address of timing 
                        
getcmd                  rdlong  tmp1, cmdpntr           wz      ' check for command
        if_z            jmp     #getcmd

checkcmd                cmp     tmp1, #1                wz      ' == 1?
        if_e            jmp     #cmdset
                        cmp     tmp1, #2                wz      ' == 2?
        if_e            jmp     #cmdblink              

cmddone                 mov     tmp1, #0                        ' clear old command
                        wrlong  tmp1, cmdpntr
                        jmp     #getcmd


                        
' turn specified LED pin on or off

cmdset                  rdlong  tmp1, pinpntr                   ' read pin to use
                        mov     pinmask, #1                     ' create mask
                        shl     pinmask, tmp1
                        rdlong  tmp1, pulsepntr                 ' get state
                        test    tmp1, #1                wc      ' tmp1.0 --> C
        if_c            or      outa, pinmask                   ' pin on if C == 1
        if_nc           andn    outa, pinmask                   ' pin off if C == 0
                        or      dira, pinmask                   ' make pin an output

                        jmp     #cmddone



' blink specified pin

cmdblink                rdlong  tmp1, pinpntr                   ' read pin to use
                        mov     pinmask, #1                     ' create mask
                        shl     pinmask, tmp1
                        rdlong  count, pulsepntr                ' read pulses
                        shl     count, #1                       ' convert to half pulses
                        rdlong  pulsetime, timepntr             ' read pulse timing
                        shr     pulsetime, #1           wz      ' convert to half pulse timing
        if_z            jmp     #cmddone

                        or      dira, pinmask                   ' make pin an output
:loop                   xor     outa, pinmask                   ' toggel the pin
                        mov     delay, pulsetime                ' set timing
                        call    #msdelay
                        djnz    count, #:loop                        

                        jmp     #cmddone 

                        

' delay in milliseconds
' -- pass timing in "delay"

msdelay                 mov     timer, ms001                    ' set for 1ms
                        add     timer, cnt                      ' sync with timer
:loop                   waitcnt timer, ms001                    ' wait, reset
                        djnz    delay, #:loop
msdelay_ret             ret

' --------------------------------------------------------------------------------------------------

ms001                   res     1                               ' clock ticks per millisecond
delay                   res     1
timer                   res     1

cmdpntr                 res     1                               ' hub address of command
pinpntr                 res     1
pulsepntr               res     1
timepntr                res     1

pinmask                 res     1
count                   res     1
pulsetime               res     1

tmp1                    res     1                               ' work vars
tmp2                    res     1

                        fit     492                                    

                        
dat

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}  